﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using System.Net;
using System.IO;

namespace Dictionary
{
    class CJp_En : CSearch
    {
        public CJp_En(int mResultNumber, SearchMode_Goo mMode)
        {
            ResultNumbe = mResultNumber;
            Mode = mMode;
        }

        /// <summary>
        /// 結果の表示件数を指定する
        /// </summary>
        private int ResultNumbe;

        /// <summary>
        /// 検索モードを指定する
        /// </summary>
        private SearchMode_Goo Mode = SearchMode_Goo.Complete;

        override public string Get(string SearchText)
        {
            SetNotice("ファイル取得開始");

            string SavePath = GetExePath() + @"\data\jp_en\" + SearchText + 
                "_" + ResultNumbe.ToString() + "_" + Mode.ToString() +".html";

            /* ファイルが存在するか確認する */
            if (!File.Exists(SavePath))
            {
                string Output;
                string Buffer;

                try
                {
                    /* 無かったらダウンロード */
                    Buffer = HTTPDownload("http://dictionary.goo.ne.jp/search.php?kind=je&mode=" + (int)Mode + "&MT=" +
                        HttpUtility.UrlEncode(SearchText, Encoding.GetEncoding("EUC-JP")));
                }
                catch
                {
                    SetNotice("ネットワークエラーが発生しました。");
                    return Page_Error;
                }

                /* 結果が無かったら、エラーを出す */
                if (Buffer.IndexOf("検索結果に該当するものが見当たりません。") != -1)
                {
                    SetNotice("検索結果に該当するものが存在しません。");
                    return Page_Error;
                }

                Output = Analyze(Buffer);

                /* 複数のページが存在しないか確認する。 */
                for (int i = 1 ; i < ResultNumbe ; i++)
                {
                    if (Buffer.IndexOf("&row=" + i.ToString()) != -1)
                    {
                        string Buffer2 = HTTPDownload("http://dictionary.goo.ne.jp/search.php?kind=je&base=1&mode="
                            + (int)Mode + "&row=" + i.ToString() + "&MT=" + HttpUtility.UrlEncode(SearchText, Encoding.GetEncoding("EUC-JP")));

                        Output += Analyze(Buffer2);
                    }
                }
                SaveFile(SavePath, Output);

                SetNotice("データキャッシュ完了");
            }
            else
            {
                SetNotice("キャッシュより取得");
            }

            return SavePath;
        }

        private string Analyze(string Buffer)
        {
            string Output;

            /* 大まかに結果部分のみ切り出し */
            Buffer = Buffer.Substring(
                Buffer.IndexOf("<!--/result-->"),
                Buffer.IndexOf("凡例はこちら</a></font>") - Buffer.IndexOf("<!--/result-->") + "凡例はこちら</a></font>".Length
                );

            Buffer += "</div>";

            int StartImageTag;
            int EndImageTag;
            string DownloadPath;

            StartImageTag = 0;

            /* imageをダウンロード */
            while (Buffer.IndexOf("<img src=\"", StartImageTag) != -1)
            {
                StartImageTag = Buffer.IndexOf("<img src=\"", StartImageTag) + "<img src=\"".Length;
                EndImageTag = Buffer.IndexOf("\"", StartImageTag);

                DownloadPath = Buffer.Substring(StartImageTag, EndImageTag - StartImageTag);

                HTTPFileDownload("http://dictionary.goo.ne.jp" + DownloadPath, GetExePath() + "\\data\\jp_en" + DownloadPath);
            }

            /* imageを参照できるようにする */
            Buffer = Buffer.Replace("src=\"/", "src=\"./");

            Output = Buffer;

            return Output;
        }
    }
}
