﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web;
using System.Net;
using System.IO;

namespace Dictionary
{
    abstract public class CSearch
    {
        /// <summary>
        /// データの取得を行う
        /// </summary>
        /// <param name="SearchText"></param>
        /// <returns>保存先のURL</returns>
        abstract public string Get(string SearchText);

        /// <summary>
        /// ファイルの保存
        /// </summary>
        /// <param name="Path"></param>
        /// <param name="Buffer"></param>
        protected void SaveFile(string Path, string Buffer)
        {
            SetNotice("ファイル書き込み開始");

            StreamWriter strm = new StreamWriter(Path, false, Encoding.GetEncoding("EUC-JP"));
            strm.Write(Buffer);
            strm.Close();

            SetNotice("ファイル書き込み終了");
        }

        /// <summary>
        /// 実行ファイルのパス
        /// </summary>
        /// <returns></returns>
        protected string GetExePath()
        {
            return Path.GetDirectoryName(Application.ExecutablePath);
        }

        /// <summary>
        /// エラーページの取得
        /// </summary>
        protected string Page_Error
        {
            get
            {
                return GetExePath() + "\\error.html";
            }
        }

        /// <summary>
        /// HTTPページのダウンロードのデフォルトを指定したもの
        /// </summary>
        /// <param name="URL"></param>
        /// <returns></returns>
        protected string HTTPDownload(string URL)
        {
            return HTTPDownload(URL, Encoding.GetEncoding("EUC-JP"));
        }

        /// <summary>
        /// HTTPページのダウンロード
        /// </summary>
        /// <param name="URL"></param>
        /// <returns></returns>
        protected string HTTPDownload(string URL,Encoding enc)
        {
            SetNotice("HTTP ページダウンロード開始");
            SetNotice("対象のアドレス：" + URL);

            WebClient wc = new WebClient();
            Stream st = wc.OpenRead(URL);

            StreamReader sr = new StreamReader(st, enc);
            string Buffer = sr.ReadToEnd();

            SetNotice("HTTP ページダウンロード完了");

            return Buffer;
        }

        /// <summary>
        /// HTTPでファイルのダウンロードを行う
        /// </summary>
        /// <param name="URL"></param>
        /// <param name="SavePath"></param>
        protected void HTTPFileDownload(string URL, string SavePath)
        {
            SetNotice("HTTP ファイルダウンロード開始");
            SetNotice("対象のアドレス：" + URL);

            if (File.Exists(SavePath))
            {
                SetNotice("ファイルキャッシュより取得");
                return;
            }

            string url = URL;
            string fileName = SavePath;

            /* WebRepuestの作成 */
            System.Net.HttpWebRequest webreq = (System.Net.HttpWebRequest)System.Net.WebRequest.Create(url);
            System.Net.HttpWebResponse webres = (System.Net.HttpWebResponse)webreq.GetResponse();

            System.IO.Stream strm = webres.GetResponseStream();

            /* Streamの作成 */
            System.IO.FileStream fs = new System.IO.FileStream(fileName,System.IO.FileMode.Create,System.IO.FileAccess.Write);

            /* ファイル書き込み */
            int i;
            while ((i = strm.ReadByte()) != -1)
            {
                fs.WriteByte(Convert.ToByte(i));
            }

            /* 解放 */
            fs.Close();
            strm.Close();

            SetNotice("HTTP ファイルダウンロード完了");
        }

        /// <summary>
        /// Notice用のテキストボックスに文字列を追加する
        /// </summary>
        /// <param name="text"></param>
        public void SetNotice(string text)
        {
            /* 一番上に最新のものがくるようにする */
            string temp = NoticeArea.Text;
            NoticeArea.Text = (text + "\r\n");
            NoticeArea.Text += temp;
        }

        /// <summary>
        /// Notice用のテキストボックスを設定する
        /// </summary>
        public void SetNoticeArea(TextBox t)
        {
            NoticeArea = t;
        }

        /// <summary>
        /// Notice用のテキストボックス
        /// </summary>
        protected TextBox NoticeArea;
    }

    public enum SearchMode_Goo
    {
        /// <summary>
        /// 前方一致
        /// </summary>
        Before = 0,

        /// <summary>
        /// 後方一致
        /// </summary>
        After = 2,

        /// <summary>
        /// 完全一致
        /// </summary>
        Complete = 1,

        /// <summary>
        /// 説明文
        /// </summary>
        Explain = 3
    }
}
